/***************************************************************************
                         dateplugin.cpp  -  description
                             -------------------
    begin                : Mon Feb 02 2004
    copyright            : (C) 2004 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "dateplugin.h"

// Qt includes
#include <qdatetime.h>

// KDE includes
#include <kfileitem.h>
#include <kio/netaccess.h>
#include <klocale.h>

DatePlugin::DatePlugin()
 : FilePlugin(0)
{
    keys.append("date");
    keys.append("year");
    keys.append("month");
    keys.append("day");
    keys.append("time");
    keys.append("hour");
    keys.append("minute");
    keys.append("second");
    keys.append("user");
    keys.append("group");
    keys.append("creationdate");
    keys.append("modificationdate");
    keys.append("accessdate");
    
 
    m_icon = "clock";   
}


QString DatePlugin::processFile(BatchRenamer* b, int i, QString token, int )
{
    if( m_replace_map.contains( token.lower() ) )
        token = m_replace_map[token.lower()];    
    else
        token = token.lower();
   
    if( !supports( token ) )
        return QString::null;
        
    QDate d = QDate::currentDate();
    QTime t = QTime::currentTime();
    QString tmp, text;

    if( token == "date" ) {
        return QDateTime::currentDateTime().toString( "dd-MM-yyyy" );
    } else if( token == "year" )
        return QString( "%1" ).arg( d.year() );
    else if( token == "month" )
        return tmp.sprintf("%0*i", 2, d.month() );
    else if( token == "day" )
        return tmp.sprintf("%0*i", 2, d.day() );
    else if( token == "time" )
        return QString( "%1-%2-%3" ).arg( t.hour() ).arg( tmp.sprintf("%0*i", 2, t.minute() ) ).arg( tmp.sprintf("%0*i", 2, t.second() ) );
    else if( token == "hour" )
        return tmp.sprintf("%0*i", 2, t.hour() );
    else if( token == "minute" )
        return tmp.sprintf("%0*i", 2, t.minute() );
    else if( token == "second" )
        return tmp.sprintf("%0*i", 2, t.second() );
    else {
        KIO::UDSEntry entry;
        KIO::NetAccess::stat( b->files()[i].src.url, entry );
        KFileItem item( entry, b->files()[i].src.url );
        if( token == "user" )
            return item.user();
        else if( token == "group" )
            return item.group();
        else if( token == "creationdate" )
            return time( item.time( KIO::UDS_CREATION_TIME ) );
        else if( token == "modificationdate" )
            return time( item.time( KIO::UDS_MODIFICATION_TIME ) );
        else if( token == "accessdate" )
            return time( item.time( KIO::UDS_ACCESS_TIME ) );
        
    }
    
    return QString::null;    
}

const QString DatePlugin::getAccelName() const
{
    return i18n("&System Functions");
}

const QString DatePlugin::getName() const
{
    return i18n("System Functions");
}

const QString DatePlugin::getPattern() const
{
    return "krename";
}

const QString DatePlugin::time( time_t time )
{
    QDateTime dt;
    dt.setTime_t( time );
    return dt.toString( "dd-MM-yyyy" );
}
