/***************************************************************************
                          krenameimpl.h  -  description
                             -------------------
    begin                : Die Mai 20 2003
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KRENAMEIMPL_H
#define KRENAMEIMPL_H

#include <qobject.h>

// Own includes
#include "batchrenamer.h"
#include "krenamedcop.h"

class HelpDialog;
class KComboBox;
class KJanusWidget;
class KMyHistoryCombo;
class KMyListBox;
class KMyListView;
class KMenuBar;
class KPushButton;
class KToggleAction;
class KURL;
class KURLRequester;
class MyHelpCombo;
class MyLabel;
class PluginLoader;
class QButtonGroup;
class QCheckBox;
class QGroupBox;
class QFileInfo;
class QFrame;
class QHBoxLayout;
class QLabel;
class QListView;
class QPushButton;
class QRect;
class QTabWidget;
class QRadioButton;
class QVBoxLayout;
class QVGroupBox;
class QWidget;
class KRenameImpl : public QObject, public KRenameDCOP {
    Q_OBJECT
    public: 
        KRenameImpl( QWidget* p, KMenuBar* m, QPushButton* finish );
        ~KRenameImpl();

        /*
         * public because both get called from KRenameImpl::launch()
         */
        void updatePre();
        void addFileOrDir( KURL name );

        void setup();
        void changeParent( QWidget* p, KMenuBar* m, QPushButton* finish, QRect r );
        static QWidget* launch( QRect rect, const KURL::List & list, KRenameImpl* impl = 0 );

        /** DCOP functions we have to implement
          */
        void addFileOrDir( QString name );
        void setFileNameTemplate( const QString & t );
        const QString fileNameTemplate() const;

        void setExtensionTemplate( const QString & t );
        const QString extensionTemplate() const;

        void setUseExtension( bool b );
        bool useExtension() const;
        
        QStringList tokens() const;

    signals:
        void pageDone( QWidget* page, const QString & title );
        void showPage( int page );
        void enableFinish( bool b );

    private slots:
        void about();
        void addDir();
        void addDirName();
        void addFile();
        bool checkErrors();
        void clearList();
        void enableControls();
        void toggleImagePreview();
        void moveUp();
        void moveDown();
        void help();
        void removeFile();
        void removeFile( int index );
        void preferences();
        void replace();
        void saveConfig();
        void start();
        void updateCount();
        void updatePreview();
        void undo();
        void changeUndoScriptName();
        void pluginHelpChanged();
        void getCoordinates();
        void changeNumbers();
        void updateDots();

        void showTokenHelp();
        void toggleName();

    private:
        static bool noRealTimePreview;

        void loadConfig();
        void fillStructures( BatchRenamer* b );
        void setupBatchRenamer( BatchRenamer* b, bool preview );
        void splitFilename( QFileInfo* fi, QString* base, QString* extension );

        void setupActions();
        void setupPage1();
        void setupPage2();
        void setupPage3();
        void setupPage4();
        void setupTab1();
        void setupPages();
        void updateHist();
        void parseCommandline();
        void addTitle( QWidget* p, QVBoxLayout* layout, QString title );

        QValueList<manualchanges> changes;
        void refreshColumnMode();

    protected:
        QWidget* parent;
        KMenuBar* menuBar;
        QPushButton* finishButton;

        PluginLoader* plugin;

        KJanusWidget* janus;
        QTabWidget* pluginTab;
        QWidget* page_1;
        QWidget* page_2;
        QWidget* page_3;
        QWidget* page_4;

        bool m_wizard;
        bool m_loadplugins;
        bool m_switching;
        bool m_autosize;
        int m_hist;
        int m_index;
        int m_step;

        KPushButton* buttonUp;
        KPushButton* buttonDown;
        KPushButton* buttonUp2;
        KPushButton* buttonDown2;
        KPushButton* buttonAdd;
        KPushButton* buttonAddDir;
        KPushButton* buttonAddDirName;
        KPushButton* buttonRemove;
        KPushButton* buttonRemoveAll;
        KPushButton* buttonReplace;
        KPushButton* buttonHelp;
        KPushButton* buttonMove;
        KPushButton* buttonCoord;
        KPushButton* buttonNumber;
        KPushButton* buttonSort;
        KPushButton* buttonRefresh;
        
        QLabel* description;
        QLabel* description2;
        QLabel* description3;
        QLabel* description4;

        QLabel* labelTemplate;
        QLabel* labelHelp;
        QLabel* labelCount;
        QLabel* labelPoint;

        KMyListBox* fileList;
        KMyListView* preview;
        QButtonGroup* groupOptions;

        QRadioButton* optionCopy;
        QRadioButton* optionMove;
        QRadioButton* optionRename;

        QGroupBox* groupDir;
        QGroupBox* groupExtension;
        QVGroupBox* groupUndo;

        KMyHistoryCombo* dirname;
        KURLRequester* urlrequester;
        KURLRequester* undorequester;
        KMyHistoryCombo* filename;
        KMyHistoryCombo* extemplate;

        QCheckBox* checkName;
        QCheckBox* checkExtension;
        QCheckBox* checkOverwrite;
        QCheckBox* checkPreview;
        QCheckBox* checkUndoScript;

        KComboBox* comboExtension;

        MyHelpCombo* comboHelp;
        HelpDialog* helpDialog;

        BatchRenamer* b;
        QValueList<int> skip;
        QValueList<replacestrings> rep;

        // ===========
        // Layout:
        // ===========

        QVBoxLayout* pageLayout;
        QVBoxLayout* pageLayout_2;
        QVBoxLayout* pageLayout_3;
        QVBoxLayout* pageLayout_4;

        QHBoxLayout* tabLayout_0;
        QHBoxLayout* tabLayout_1;
        QHBoxLayout* tabLayout_2;
        QVBoxLayout* tabLayout_3;
        QVBoxLayout* groupAdvancedExtensionLayout;
        QVBoxLayout* groupOptionsLayout;
        QHBoxLayout* groupDirLayout;
        QVBoxLayout* groupNumberLayout;
        QHBoxLayout* groupExtensionLayout;

        // page1
        QHBoxLayout* Layout2;
        QVBoxLayout* Layout3;
        QHBoxLayout* Layout4;
        QVBoxLayout* Layout5;

        // page4
        QVBoxLayout* Layout10;
        QHBoxLayout* Layout13;
        QHBoxLayout* Layout15;
        QVBoxLayout* Layout16;
        QHBoxLayout* Layout18;
        QVBoxLayout* Layout21;
        QHBoxLayout* Layout22;
        QVBoxLayout* Layout23;

        // tab
        QHBoxLayout* Layout100;
        QHBoxLayout* Layout101;
};

#endif
