/***************************************************************************
                          pluginloader.cpp  -  description
                             -------------------
    begin                : Sun Dec 30 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 * Parts of this code are copied from
 * Franz Schmid's <Franz.Schmid@altmuehlnet.de>
 * Scribus.
 */

// Own includes
#include "pluginloader.h"
#include "fileplugin.h"
#include "datetime.h"
#include "mydirplugin.h"
#include "permission.h"
#include "pictureplugin.h"
#include "commandplugin.h"
#include "dateplugin.h"

// QT includes
#include <qstringlist.h>

// KDE includes
#include <kapplication.h>
#include <kservice.h>

/* use a prime number here */
#define DICT_SIZE 43

PluginLoader::PluginLoader()
{
    libs.setAutoDelete( true );
    m_loaded = false;
}

PluginLoader::~PluginLoader()
{ }

PluginLoader* PluginLoader::instance()
{
    if( !m_plugin )
        m_plugin = new PluginLoader();

    return m_plugin;
}

PluginLoader* PluginLoader::m_plugin = 0;

void PluginLoader::loadPlugins( bool fileplugins )
{
    // do not load plugins twice
    if( m_loaded )
        return;

    m_loaded = true;

    addPlugin( new CommandPlugin() );
    addPlugin( new MyPermPlugin() );
    addPlugin( new MyDatePlugin() );
    addPlugin( new MyDirPlugin() );
    addPlugin( new PicturePlugin() );
    addPlugin( new DatePlugin() );
    
    if( fileplugins )
        loadFilePlugins();
        
    QPtrListIterator<PluginLoader::PluginLibrary> it( libs );
    for( ; it.current(); ++it ) {
        if( (*it)->plugin->type() == TYPE_BRACKET ) {
            const QStringList list = (*it)->plugin->getKeys();
            for( unsigned int i = 0; i < list.count(); i++ ) {
                if( m_bracket_map.contains( list[i].lower() ) ) {
                    if( (*it)->plugin->getPattern().isEmpty() )
                        qDebug( (*it)->plugin->getName() );
                    m_bracket_map.insert( (*it)->plugin->getPattern() + list[i].lower(), (*it)->plugin );
                    (*it)->plugin->replace( (*it)->plugin->getPattern() + list[i].lower(), list[i] );
                } else
                    m_bracket_map.insert( list[i].lower(), (*it)->plugin );
            }
        }
    }

}

Plugin* PluginLoader::findPlugin( const QString & token )
{
    
    if( m_bracket_map.contains( token.lower() ) )
        return m_bracket_map[token.lower()];
    else
        return 0;
}

void PluginLoader::addPlugin( Plugin* plugin )
{
    PluginLibrary* listitem=new PluginLibrary;
    listitem->plugin = plugin;
    listitem->check = 0;
    listitem->usePlugin = plugin->alwaysUsed();
    libs.append( listitem );
}

void PluginLoader::loadFilePlugins()
{
    KService::List list = KService::allServices();
    for( unsigned int i = 0; i < list.count(); i++ ) {
        KService* s = (KService*)list[i];
        if( !s->terminal() && s->type() == "Service" && s->hasServiceType( "KFilePlugin" ) ) {
            FilePlugin* kfileplugin = new FilePlugin( s );
            if( kfileplugin->isValid() ) {
                addPlugin( kfileplugin );
                kapp->processEvents();
            } else
                delete kfileplugin;
        }
    }
}
